/*
 * Copyright (c) 2008 Jiri Svoboda
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup debug
 * @{
 */
/** @file
 */

#include <stdio.h>
#include <assert.h>
#include <errno.h>
#include <sys/types.h>
#include <udebug.h>

#include "../../main.h"
#include "../../cons.h"
#include "bstore.h"

void bstore_initialize(bstore_t *bs)
{
	bs->valid = false;
}

int bstore_save(bstore_t *bs, uintptr_t addr)
{
	uint32_t v;
	int rc;

	assert(bs->valid == false);

	rc = udebug_mem_read(app_phone, &v, addr, sizeof(uint32_t));
	if (rc == EOK) {
		bs->valid = true;
		bs->address = addr;
		bs->value = v;
	} else {
		cons_printf("Error reading memory address 0x%zx\n", addr);
	}

	return rc;
}

int bstore_push(bstore_t *bs, uintptr_t addr, uint32_t value)
{
	int rc;

	rc = bstore_save(bs, addr);
	if (rc != EOK) return rc;

	rc = udebug_mem_write(app_phone, &value, addr, sizeof(uint32_t));
	if (rc != EOK) {
		cons_printf("Error writing memory address 0x%zx\n", addr);
		bstore_empty(bs);
		return rc;
	}

	return EOK;
}

int bstore_pop(bstore_t *bs)
{
	int rc;

	assert(bs->valid == true);
	rc = udebug_mem_write(app_phone, &bs->value, bs->address,
	    sizeof(uint32_t));

	if (rc == EOK) {
		bs->valid = false;
	} else {
		cons_printf("Error writing memory address 0x%zx\n",
		    bs->address);
	}

	return rc;
}

void bstore_empty(bstore_t *bs)
{
	assert(bs->valid == true);
	bs->valid = false;
}

/** @}
 */
